#! /usr/bin/perl -w

# mailtools 0.2
#
# Tools for analysing mbox format mail spools.
#
# Copyright (C) 2000 Alan De Smet chaos@highprogrammer.com
# http://www.highprogrammer.com/alan/perl/#mailtools
#
# This software is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program in file entitled "COPYING"; if
# not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA  02111-1307, USA.  A copy of the
# GNU General Public License is available at
# http://www.fsf.org/copyleft/gpl.html .
#

use strict;

package Mail::Folder::MBox;

sub new {
	my $class = shift;
	$class = ref($class) if ref($class);
	my $filename = shift;
	my $filehandle  = do { local *FILE };
	open($filehandle, $filename) or return undef;

	bless my $self = {
		'filehandle' => $filehandle,
		'nextline' => undef,
		'open' => 1,
	}, $class;

	$self->next_line();

	return $self;
}

sub validate {
	my($self) = shift;
	if(not defined $self or not defined $self->{'open'}) {
		die "Attempted to use invalid Mail::Folder::MBox.\n";
	}
}

sub next_line {
	my($self) = shift;
	$self->validate();
	my($previous) = $self->{'nextline'};
	my($fh) = $self->{'filehandle'};
	$self->{'nextline'} = <$fh>;
	return $previous;
}

sub peek_next_line {
	my($self) = shift;
	$self->validate();
	return $self->{'nextline'};
}

sub next_message {
	my($self) = shift;
	my(@message);
	my($previous_line) = 'X'; # initialize to defined, non-blank

	if(not defined $self->peek_next_line()) {
		return wantarray ? () : undef;
	}

	while(1)
	{
		last if not defined $self->peek_next_line();
		if($previous_line =~ /^\n$/) {
			last if $self->peek_next_line() =~ /^From /;
		}
		$previous_line = $self->next_line();
		push @message, $previous_line;
	}

	return wantarray ? @message : join('', @message);
}

1;
