#! /usr/bin/perl -w

# mailtools 0.2
#
# Tools for analysing mbox format mail spools.
#
# Copyright (C) 2000 Alan De Smet chaos@highprogrammer.com
# http://www.highprogrammer.com/alan/perl/#mailtools
#
# This software is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program in file entitled "COPYING"; if
# not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA  02111-1307, USA.  A copy of the
# GNU General Public License is available at
# http://www.fsf.org/copyleft/gpl.html .
#

use strict;

#===============================================================================
package Mail::ThreadNode;

sub new {
	my $class = shift;
	$class = ref($class) if ref($class);
	my($parent) = shift;

	bless my $self = {
		'message' => undef,
		'parent' => $parent,
		'children' => {},
	}, $class;

	return $self;
}

sub debug_dump {
	my($self) = shift;
	my($subject) = ('**Lost Message**');
	if(defined $self->message()) {
		$subject = $self->message()->subject();
	}

	if(not defined $subject) {
		print STDERR "Missing subject\n";
		$subject = '**Subject Lost**';
	}

	my(@return);

	foreach my $child ( values %{ $self->{'children'} } ) {
		my(@child_return) = $child->debug_dump();
		push @return, @child_return;
	}
	map s/^/  /, @return;
	unshift @return, "$subject\n";
	return @return;
}

sub message {
	my($self) = shift;
	my($assign) = shift;
	$self->{'message'} = $assign if defined $assign;
	return $self->{'message'};
}

sub parent {
	my($self) = shift;
	#my($assign) = shift;
	#$self->{'parent'} = $assign if defined $assign;
	return $self->{'parent'};
}

sub children {
	my($self) = shift;
	#my($assign) = shift;
	#$self->{'child'} = $assign if defined $assign;
	return (values %{$self->{'children'}});
}

# PRIVATE
sub forget_child {
	my($self) = shift;
	my($child) = shift;
	if(not defined $self->{'children'}{$child}) {
		print STDERR "ERROR: Can't delete child?\n";
		return;
	}
	delete $self->{'children'}{$child};
}

# PRIVATE
sub discover_child {
	my($self) = shift;
	my($child) = shift;
	if($self == $child) {
		die;
	}
	if(defined $self->{'children'}{$child}) {
		print STDERR "ERROR: Child alread exists\n";
		return;
	}
	$self->{'children'}{$child} = $child;
}


# PRIVATE
sub can_reach {
	my($self) = shift;
	my($potential_child) = shift;
	foreach my $child ( %{ $self->{'children'} } ) {
		if($child == $potential_child) {
			return 1;
		}
		if($child->can_reach($potential_child)) {
			return 1;
		}
	}
	return 0;
}

sub reparent {
	my($self) = shift;
	my($parent) = shift;

	if(not defined $self and not defined $parent) {
		die;
	}

	if(defined $self and defined $parent and $self == $parent) {
		die;
	}

	# If I have an existing parent, bid them farewell
	my($old_parent) = $self->{'parent'};
	if(defined $old_parent) {
		$old_parent->forget_child($self);
	}

	$self->{'parent'} = $parent;
	if(defined $parent) {
		$parent->discover_child($self);
	}
}


1;
